@@ -78,7 +78,7 @@ struct options
     int amode;                  /* attack mode          */
     int essid_set;              /* essid set flag       */
     int bssid_set;              /* bssid set flag       */
-    char essid[33];             /* target ESSID         */
+    uchar essid[33];            /* target ESSID         */
     uchar bssid[6];             /* target BSSID         */
     int nbcpu;                  /* # of cracker threads */
     int is_quiet;               /* quiet mode flag      */
@@ -131,7 +131,7 @@ struct AP_info
 {
     struct AP_info *next;       /* next AP in linked list       */
     uchar bssid[6];             /* access point MAC address     */
-    char essid[33];             /* access point identifier      */
+    uchar essid[33];            /* access point identifier      */
     uchar lanip[4];             /* IP address if unencrypted    */
     uchar *ivbuf;               /* table holding WEP IV data    */
     uchar **uiv_root;           /* IV uniqueness root struct    */
@@ -687,25 +687,29 @@ skip_station:
 
         if( h80211[z] != h80211[z + 1] || h80211[z + 2] != 0x03 )
         {
-            ap_cur->crypt = 2;          /* encryption = WEP */
-
             /* check the extended IV flag */
 
-            if( ( h80211[z + 3] & 0x20 ) != 0 )
-                ap_cur->crypt = 3;      /* encryption = WPA */
+            if( ( h80211[z + 3] & 0x20 ) == 0 )
+            {
+                /* check the WEP key index */
 
-            /* check the WEP key index */
+                if( opt.index != 0 &&
+                    ( h80211[z + 3] >> 6 ) != opt.index - 1 )
+                    goto unlock_mx_apl;
 
-            if( opt.index != 0 &&
-                ( h80211[z + 3] >> 6 ) != opt.index - 1 )
-                goto unlock_mx_apl;
+                /* save the IV & first two output bytes */
 
-            /* save the IV & first two output bytes */
+                memcpy( buffer    , h80211 + z    , 3 );
+                memcpy( buffer + 3, h80211 + z + 4, 2 );
 
-            memcpy( buffer    , h80211 + z    , 3 );
-            memcpy( buffer + 3, h80211 + z + 4, 2 );
+                ap_cur->crypt = 2;  /* encryption = WEP */
 
-            goto add_wep_iv;
+                goto add_wep_iv;
+            }
+
+            ap_cur->crypt = 3;      /* encryption = WPA */
+
+            goto unlock_mx_apl;
         }
 
         if( ap_cur->crypt < 0 )
@@ -932,7 +936,7 @@ int crack_wep_thread( void *arg )
 
             for( i = j = 0; i < q; i++ )
             {
-                jj[i] = j = ( j + S[i] + K[i % (3 + opt.keylen)] ) & 0xFF;
+                jj[i] = j = ( j + S[i] + K[i & (2 + opt.keylen)] ) & 0xFF;
                 SWAP( S[i], S[j] );
             }
 
@@ -1142,8 +1146,7 @@ void show_wep_stats( int B, int force )
     if( opt.l33t )
         printf( "\33[34;1m" );
 
-    printf( "\33[2;%dHaircrack %d.%d\n\n", (ws.ws_col - 12) / 2,
-            _VER_MAJOR, _VER_MINOR );
+    printf( "\33[2;%dHaircrack 2.2\n\n", (ws.ws_col - 12) / 2 );
 
     if( opt.l33t )
         printf( "\33[33;1m" );
@@ -1210,7 +1213,6 @@ int check_wep_key( int B )
     uchar x1, x2;
     unsigned long xv;
     int i, j, n, bad;
-    int nb_ascii;
 
     nb_tried++;
     bad = 0;
@@ -1229,7 +1231,7 @@ int check_wep_key( int B )
 
         for( i = j = 0; i < 256; i++ )
         {
-            j = ( j + S[i] + K[i % (3 + opt.keylen)]) & 0xFF;
+            j = ( j + S[i] + K[i & (2 + opt.keylen)]) & 0xFF;
             SWAP( S[i], S[j] );
         }
 
@@ -1248,13 +1250,6 @@ int check_wep_key( int B )
             return( FAILURE );
     }
 
-    nb_ascii = 0;
-
-    for( i = 0; i < opt.keylen; i++ )
-        if(   wep.key[i] == 0 ||
-            ( wep.key[i] >= 32 && wep.key[i] < 127 ) )
-            nb_ascii++;
-
     if( opt.is_quiet )
         printf( "KEY FOUND! [ " );
     else
@@ -1264,36 +1259,17 @@ int check_wep_key( int B )
         if( opt.l33t )
             printf( "\33[31;1m" );
 
-        n = ( 80 - 14 - opt.keylen * 3 ) / 2;
-
-        if( 100 * nb_ascii > 75 * opt.keylen )
-            n -= ( opt.keylen + 4 ) / 2;
-
-        if( n <= 0 ) n = 0;
-
-        printf( "\33[K\33[%dCKEY FOUND! [ ", n );
+        printf( "\33[K\33[%dCKEY FOUND! [ ",
+                ( 80 - 14 - opt.keylen * 3 ) / 2 );
     }
 
     for( i = 0; i < opt.keylen - 1; i++ )
         printf( "%02X:", wep.key[i] );
-    printf( "%02X ] ",   wep.key[i] );
-
-    if( 100 * nb_ascii > 75 * opt.keylen )
-    {
-        printf( "(" );
-
-        for( i = 0; i < opt.keylen; i++ )
-            printf( "%c", ( ( wep.key[i] >  31 && wep.key[i] < 127 ) ||
-                              wep.key[i] > 160 ) ? wep.key[i] : '.' );
-
-        printf( ")" );
-    }
+    printf( "%02X ]\n",  wep.key[i] );
 
     if( opt.l33t )
         printf( "\33[32;22m" );
 
-    printf( "\n" );
-
     return( SUCCESS );
 }
 
@@ -1364,7 +1340,7 @@ int calc_poll( int B )
 
     if( opt.is_alnum )
     {
-        for( i = 1; i < 32; i++ )
+        for( i = 0; i < 32; i++ )
             wep.poll[B][i].val = -1;
 
         for( i = 127; i < 256; i++ )
@@ -1655,10 +1631,10 @@ int do_wep_crack2( int B )
 
 /* derive the PMK from the passphrase and the essid */
 
-void calc_pmk( char *key, char *essid, uchar pmk[40] )
+void calc_pmk( uchar *key, uchar *essid, uchar pmk[40] )
 {
     int i, j, slen;
-    uchar buffer[65];
+    uchar buffer[64];
     sha1_context ctx_ipad;
     sha1_context ctx_opad;
     sha1_context sha1_ctx;
@@ -1667,8 +1643,8 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
 
     /* setup the inner and outer contexts */
 
-    memset( buffer, 0, sizeof( buffer ) );
-    strncpy( (char *) buffer, key, sizeof( buffer ) - 1 );
+    memset( buffer, 0, 64 );
+    strcpy( buffer, key );
 
     for( i = 0; i < 64; i++ )
         buffer[i] ^= 0x36;
@@ -1685,8 +1661,7 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
     /* iterate HMAC-SHA1 over itself 8192 times */
 
     essid[slen - 1] = '\1';
-    hmac_sha1( (uchar *) key, strlen( key ),
-               (uchar *) essid, slen, pmk );
+    hmac_sha1( key, strlen( key ), essid, slen, pmk );
     memcpy( buffer, pmk, 20 );
 
     for( i = 1; i < 4096; i++ )
@@ -1704,8 +1679,7 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
     }
 
     essid[slen - 1] = '\2';
-    hmac_sha1( (uchar *) key, strlen( key ),
-               (uchar *) essid, slen, pmk + 20 );
+    hmac_sha1( key, strlen( key ), essid, slen, pmk + 20 );
     memcpy( buffer, pmk + 20, 20 );
 
     for( i = 1; i < 4096; i++ )
@@ -1727,9 +1701,9 @@ void calc_pmk( char *key, char *essid, uchar pmk[40] )
 
 int crack_wpa_thread( void *arg )
 {
-    char  essid[36];
-    char  key1[128], key2[128];
-    uchar pmk1[128], pmk2[128];
+    uchar essid[36];
+    uchar key1[128], pmk1[40];
+    uchar key2[128], pmk2[40];
 
 #ifdef __i386__
 
@@ -1825,11 +1799,8 @@ int crack_wpa_thread( void *arg )
 
         essid[slen - 1] = '\1';
 
-        hmac_sha1( (uchar *) key1, strlen( key1 ),
-                   (uchar *) essid, slen,  pmk1 );
-
-        hmac_sha1( (uchar *) key2, strlen( key2 ),
-                   (uchar *) essid, slen,  pmk2 );
+        hmac_sha1( key1, strlen( key1 ), essid, slen, pmk1 );
+        hmac_sha1( key2, strlen( key2 ), essid, slen, pmk2 );
 
         u = (uint *) pmk1;
         v = (uint *) pmk2;
@@ -1866,11 +1837,8 @@ int crack_wpa_thread( void *arg )
 
         essid[slen - 1] = '\2';
 
-        hmac_sha1( (uchar *) key1, strlen( key1 ),
-                   (uchar *) essid, slen,  pmk1 + 20 );
-
-        hmac_sha1( (uchar *) key2, strlen( key2 ),
-                   (uchar *) essid, slen,  pmk2 + 20 );
+        hmac_sha1( key1, strlen( key1 ), essid, slen, pmk1 + 20 );
+        hmac_sha1( key2, strlen( key2 ), essid, slen, pmk2 + 20 );
 
         u = (uint *) ( pmk1 + 20 );
         v = (uint *) ( pmk2 + 20 );
@@ -1926,7 +1894,7 @@ int crack_wpa_thread( void *arg )
 
 /* display the current wpa key info, matrix-like */
 
-void show_wpa_stats( char *key, uchar pmk[32], uchar ptk[64],
+void show_wpa_stats( uchar *key, uchar pmk[32], uchar ptk[64],
                      uchar mic[16], int force )
 {
     float delta;
@@ -1960,8 +1928,7 @@ void show_wpa_stats( char *key, uchar pmk[32], uchar ptk[64],
             nb_tried, (float) nb_kprev / delta );
 
     memset( tmpbuf, ' ', sizeof( tmpbuf ) );
-    memcpy( tmpbuf, key, strlen( key ) > 27 ? 27 :
-                         strlen( key ) );
+    memcpy( tmpbuf, key, strlen( key ) > 27 ? 27 : strlen( key ) );
     tmpbuf[27] = '\0';
 
     if( opt.l33t ) printf( "\33[37;1m" );
@@ -2000,12 +1967,12 @@ void show_wpa_stats( char *key, uchar pmk[32], uchar ptk[64],
 int do_wpa_crack( struct AP_info *ap )
 {
     int i, cid;
-    char key1[128], key2[128];
 
     uchar pke[100];
     uchar pmk1[40], ptk1[80];
     uchar pmk2[40], ptk2[80];
-    uchar mic1[20], mic2[20];
+    uchar key1[128], mic1[20];
+    uchar key2[128], mic2[20];
 
     /* send the ESSID to each thread */
 
@@ -2058,8 +2025,7 @@ int do_wpa_crack( struct AP_info *ap )
         if( opt.l33t )
             printf( "\33[34;1m" );
 
-        printf( "\33[2;34Haircrack %d.%d",
-                _VER_MAJOR, _VER_MINOR );
+        printf( "\33[2;34Haircrack 2.2" );
     }
 
     while( 1 )
@@ -2204,6 +2170,8 @@ void sighandler( int signum )
 
 char usage[] =
 
+"\n"
+"  aircrack 2.2 - (C) 2004,2005 Christophe Devine\n"
 "\n"
 "  Common options:\n"
 "\n"
@@ -2212,21 +2180,22 @@ char usage[] =
 "      -b <bssid> : target selection: access point's MAC\n"
 "      -p <nbcpu> : SMP support: # of processes to start\n"
 "      -q         : enable quiet mode (no status output)\n"
-"      -w <words> : path to a dictionary file\n"
 "\n"
 "  Static WEP cracking options:\n"
 "\n"
 "      -c         : search alpha-numeric characters only\n"
 "      -d <start> : debug - specify beginning of the key\n"
 "      -m <maddr> : MAC address to filter usable packets\n"
-"      -n <nbits> : WEP key length: 64 / 128 / 152 / 256\n"
+"      -n <nbits> : WEP key length: 64 / 128 / 256 / 512\n"
 "      -i <index> : WEP key index (1 to 4), default: any\n"
 "      -f <fudge> : bruteforce fudge factor,  default: 2\n"
 "      -k <korek> : disable one attack method  (1 to 17)\n"
 "      -x         : don't bruteforce the last 2 keybytes\n"
 "      -y         : experimental single bruteforce attack\n"
 "\n"
-"  aircrack %d.%d - (C) 2004,2005 Christophe Devine\n"
+"  WPA-PSK cracking options:\n"
+"\n"
+"      -w <words> : path to dictionary file\n"
 "\n"
 "  usage: aircrack [options] <.cap / .ivs file(s)>\n"
 "\n";
@@ -2234,7 +2203,7 @@ char usage[] =
 int main( int argc, char *argv[] )
 {
     int i, n, ret;
-    char *s, buf[128];
+    uchar *s, buf[128];
     struct AP_info *ap_cur;
 
     ret = FAILURE;
@@ -2244,7 +2213,7 @@ int main( int argc, char *argv[] )
     if( argc < 2 )
     {
     usage:
-        printf( usage, _VER_MAJOR, _VER_MINOR );
+        printf( usage );
         return( ret );
     }
 
@@ -2275,8 +2244,7 @@ int main( int argc, char *argv[] )
 
             case 'e' :
 
-                memset(  opt.essid, 0, sizeof( opt.essid ) );
-                strncpy( opt.essid, optarg, sizeof( opt.essid ) - 1 );
+                strncpy( opt.essid, optarg, 32 );
                 opt.essid_set = 1;
                 break;
 
@@ -2411,8 +2379,7 @@ int main( int argc, char *argv[] )
 
                 if( sscanf( optarg, "%d", &opt.keylen ) != 1 ||
                     ( opt.keylen !=  64 && opt.keylen != 128 &&
-                      opt.keylen != 152 && opt.keylen != 256 &&
-                      opt.keylen != 512 ) )
+                      opt.keylen != 256 && opt.keylen != 512 ) )
                 {
                     printf( "Invalid WEP key length.\n" );
                     return( FAILURE );
@@ -2696,16 +2663,8 @@ int main( int argc, char *argv[] )
 
     signal( SIGWINCH, sighandler );
 
-    if( opt.amode == 1 )
-        goto crack_wep;
-
-    if( opt.amode == 2 )
-        goto crack_wpa;
-
-    if( ap_cur->crypt == 2 )
+    if( opt.amode == 1 || ap_cur->crypt == 2 )
     {
-    crack_wep:
-
         if( opt.keylen == 0 )
             opt.keylen = 13;
 
@@ -2776,10 +2735,8 @@ int main( int argc, char *argv[] )
         }
     }
 
-    if( ap_cur->crypt == 3 )
+    if( opt.amode == 2 || ap_cur->crypt == 3 )
     {
-    crack_wpa:
-
         if( opt.dict == NULL )
         {
             printf( "Please specify a dictionary (option -w).\n" );
@@ -2809,10 +2766,7 @@ int main( int argc, char *argv[] )
         }
 
         if( opt.essid_set && ap_cur->essid[0] == '\0' )
-        {
-            memset(  ap_cur->essid, 0, sizeof( ap_cur->essid ) );
-            strncpy( ap_cur->essid, opt.essid, sizeof( ap_cur->essid ) - 1 );
-        }
+            strncpy( ap_cur->essid, opt.essid, 32 );
 
         for( i = 0; i < opt.nbcpu; i++ )
         {
