@@ -113,14 +113,9 @@ int dump_initialize( char *output_prefix, int ivs_only )
 
     /* create the output csv file */
 
-    if( strlen( output_prefix ) >= sizeof( o_filename ) - 5 )
-        output_prefix[sizeof( o_filename ) - 5] = '\0';
-
     if( strcmp( output_prefix, "-" ) != 0 )
     {
-        memset( o_filename, 0, sizeof( o_filename ) );
-        snprintf( o_filename,  sizeof( o_filename ) - 1,
-                  "%s.txt", output_prefix );
+        sprintf( o_filename, "%s.txt", output_prefix );
 
         if( ( f_csv_out = fopen( o_filename, "wb+" ) ) == NULL )
         {
@@ -132,16 +127,17 @@ int dump_initialize( char *output_prefix, int ivs_only )
 
     /* open or create the output packet capture file */
 
+    if( strlen( output_prefix ) >= sizeof( o_filename ) - 5 )
+    {
+        output_prefix[sizeof( o_filename ) - 5] = '\0';
+    }
+
     if( ivs_only == 0 )
     {
         n = sizeof( struct pcap_file_header );
 
         if( strcmp( output_prefix, "-" ) != 0 )
-        {
-            memset( o_filename, 0, sizeof( o_filename ) );
-            snprintf( o_filename,  sizeof( o_filename ) - 1,
-                      "%s.cap", output_prefix );
-        }
+            sprintf( o_filename, "%s.cap", output_prefix );
         else
         {
             f_cap_out = fdopen( 1, "wb" );
@@ -213,11 +209,7 @@ int dump_initialize( char *output_prefix, int ivs_only )
         memset( prev_bssid, 0, 6 );
 
         if( strcmp( output_prefix, "-" ) != 0 )
-        {
-            memset( o_filename, 0, sizeof( o_filename ) );
-            snprintf( o_filename,  sizeof( o_filename ) - 1,
-                      "%s.ivs", output_prefix );
-        }
+            sprintf( o_filename, "%s.ivs", output_prefix );
         else
         {
             f_ivs_out = fdopen( 1, "wb" );
@@ -269,8 +261,7 @@ int dump_initialize( char *output_prefix, int ivs_only )
 }
 
 int dump_add_packet( unsigned char *h80211, int caplen, int power,
-                     int channel, uint tv_sec, uint tv_usec )
-
+                     uint tv_sec, uint tv_usec )
 {
     int i, n;
 
@@ -495,8 +486,7 @@ skip_station:
                 memcpy( ap_cur->essid, p + 2, n );
 
                 for( i = 0; i < n; i++ )
-                    if( ap_cur->essid[i] < 32 ||
-                      ( ap_cur->essid[i] > 126 && ap_cur->essid[i] < 160 ) )
+                    if( ap_cur->essid[i] < ' ' )
                         ap_cur->essid[i] = '.';
             }
 
@@ -532,8 +522,7 @@ skip_station:
                 memcpy( ap_cur->essid, p + 2, n );
 
                 for( i = 0; i < n; i++ )
-                    if( ap_cur->essid[i] < 32 ||
-                      ( ap_cur->essid[i] > 126 && ap_cur->essid[i] < 160 ) )
+                    if( ap_cur->essid[i] < ' ' )
                         ap_cur->essid[i] = '.';
             }
 
@@ -543,9 +532,6 @@ skip_station:
 
     /* packet parsing: some data */
 
-    if( ap_cur->chanl == -1 )
-        ap_cur->chanl = channel;
-
     if( ( h80211[0] & 0x0C ) == 0x08 )
     {
         /* check the SNAP header to see if data is encrypted */
@@ -568,13 +554,19 @@ skip_station:
 
             if( h80211[z + 6] == 0x08 && h80211[z + 7] == 0x06 )
                 memcpy( ap_cur->lanip, &h80211[z + 22], 4 );
+
+            goto write_packet;
         }
-        else
-            ap_cur->crypt = 2 + ( ( h80211[z + 3] & 0x20 ) >> 5 );
+
+        /* packet parsing: WEP or WPA encrypted data */
 
         if( z + 10 > pkh.caplen )
             goto write_packet;
 
+        /* if the extended IV flag is set then it's WPA */
+
+        ap_cur->crypt = 2 + ( ( h80211[z + 3] & 0x20 ) >> 5 );
+
         if( ap_cur->crypt == 2 )
         {
             /* WEP: check if we've already seen this IV */
@@ -677,8 +669,8 @@ void dump_print( int ws_row, int ws_col )
 
     /* print some informations about each detected AP */
 
-    fprintf( stderr, "\n BSSID              PWR  Beacons"
-                     "   # Data  CH  MB  ENC   ESSID\n\n" );
+    fprintf( stderr, "\n  BSSID              PWR  Beacons   "
+                     "   IP / # Data  CH  MB  ENC  ESSID\n\n" );
 
     nlines = 5;
 
@@ -698,17 +690,28 @@ void dump_print( int ws_row, int ws_col )
 
         nlines++;
 
-        fprintf( stderr, " %02X:%02X:%02X:%02X:%02X:%02X",
+        fprintf( stderr, "  %02X:%02X:%02X:%02X:%02X:%02X",
                 ap_cur->bssid[0], ap_cur->bssid[1],
                 ap_cur->bssid[2], ap_cur->bssid[3],
                 ap_cur->bssid[4], ap_cur->bssid[5] );
 
-        fprintf( stderr, "  %3d %8ld %8ld",
-                 ap_cur->power,
-                 ap_cur->nb_bcn,
-                 ap_cur->nb_data );
+        fprintf( stderr, "  %3d %8ld  ", ap_cur->power, ap_cur->nb_bcn );
 
-        fprintf( stderr, " %3d %3d  ", ap_cur->chanl, ap_cur->speed );
+        if( ap_cur->crypt == 0 )
+        {
+            strbuf[0] = '\0';
+
+            if( *(unsigned long *)( ap_cur->lanip ) )
+                sprintf( strbuf, "%d.%d.%d.%3d",
+                         ap_cur->lanip[0], ap_cur->lanip[1],
+                         ap_cur->lanip[2], ap_cur->lanip[3] );
+
+            fprintf( stderr, "%15s", strbuf );
+        }
+        else
+            fprintf( stderr, "%15ld", ap_cur->nb_data );
+
+        fprintf( stderr, "  %2d %3d  ", ap_cur->chanl, ap_cur->speed );
 
         switch( ap_cur->crypt )
         {
@@ -719,11 +722,9 @@ void dump_print( int ws_row, int ws_col )
             default: fprintf( stderr, "    " ); break;
         }
 
-        memset( strbuf, 0, sizeof( strbuf ) );
-        snprintf( strbuf,  sizeof( strbuf ) - 1,
-                  "%-32s", ap_cur->essid );
-        strbuf[ws_col - 58] = '\0';
-        fprintf( stderr, "  %s\n", strbuf );
+        sprintf( strbuf, "%-32s", ap_cur->essid );
+        strbuf[ws_col - 67] = '\0';
+        fprintf( stderr, " %s\n", strbuf );
 
         ap_cur = ap_cur->prev;
     }
@@ -737,14 +738,14 @@ void dump_print( int ws_row, int ws_col )
     if( ws_row != 0 && nlines > ws_row )
         return;
 
-    memset( strbuf, 0x20, ws_col - 1 );
+    memset( strbuf, 0x20, ws_col - 2 );
     fprintf( stderr, "%s\n", strbuf );
 
-    memcpy( strbuf, " BSSID              STATION "
-            "           PWR  Packets  ESSID", 58 );
+    memcpy( strbuf, "  BSSID              STATION "
+            "           PWR   Packets  ESSID", 60 );
     fprintf( stderr, "%s\n", strbuf );
 
-    memset( strbuf, 0x20, ws_col - 1 );
+    memset( strbuf, 0x20, ws_col - 2 );
     fprintf( stderr, "%s\n", strbuf );
 
     ap_cur = ap_end;
@@ -777,7 +778,7 @@ void dump_print( int ws_row, int ws_col )
 
             nlines++;
 
-            fprintf( stderr, " %02X:%02X:%02X:%02X:%02X:%02X",
+            fprintf( stderr, "  %02X:%02X:%02X:%02X:%02X:%02X",
                     ap_cur->bssid[0], ap_cur->bssid[1],
                     ap_cur->bssid[2], ap_cur->bssid[3],
                     ap_cur->bssid[4], ap_cur->bssid[5] );
@@ -787,17 +788,11 @@ void dump_print( int ws_row, int ws_col )
                     st_cur->stmac[2], st_cur->stmac[3],
                     st_cur->stmac[4], st_cur->stmac[5] );
 
-            if( st_cur->power != -1 )
-                fprintf( stderr, "  %3d", st_cur->power );
-            else
-                fprintf( stderr, "     " );
+            fprintf( stderr, "  %3d %9ld", st_cur->power,
+                                  st_cur->nb_pkt );
 
-            fprintf( stderr, " %8ld", st_cur->nb_pkt );
-
-            memset( strbuf, 0, sizeof( strbuf ) );
-            snprintf( strbuf,  sizeof( strbuf ) - 1,
-                      "%-32s", ap_cur->essid );
-            strbuf[ws_col - 54] = '\0';
+            sprintf( strbuf, "%-32s", ap_cur->essid );
+            strbuf[ws_col - 57] = '\0';
             fprintf( stderr, "  %s\n", strbuf );
 
             st_cur = st_cur->prev;
@@ -888,7 +883,7 @@ void dump_write_csv( void )
 
         if( ap_cur->nb_pkt < 2 )
         {
-            st_cur = st_cur->next;
+            ap_cur = ap_cur->next;
             continue;
         }
 
@@ -931,8 +926,6 @@ void dump_write_csv( void )
     sync();
 }
 
-int channel;
-int ch_pipe[2];
 int do_exit = 0;
 struct winsize ws;
 
@@ -940,22 +933,28 @@ void sighandler( int signum )
 {
     signal( signum, sighandler );
 
-    if( signum == SIGUSR1 )
-        read( ch_pipe[0], &channel, sizeof( int ) );
-
     if( signum == SIGINT || signum == SIGTERM )
     {
         do_exit = 1;
         alarm( 1 );
         signal( SIGALRM, sighandler );
-        printf( "\n" );
     }
 
     if( signum == SIGALRM )
     {
+        if( f_cap_in != NULL )
+        {
+            fprintf( stderr, "\33[1;1H" );
+            dump_print( 0, ws.ws_col );
+            fprintf( stderr, "\33[J\n" );
+        }
+
+        dump_write_csv();
+
         if( f_ivs_out != NULL ) fclose( f_ivs_out );
         if( f_cap_out != NULL ) fclose( f_cap_out );
         if( f_csv_out != NULL ) fclose( f_csv_out );
+
         exit( 1 );
     }
 
@@ -970,12 +969,6 @@ void sighandler( int signum )
             ws.ws_col = 80;
         }
 
-        if( ws.ws_col < 54 )
-            ws.ws_col = 54;
-
-        if( ws.ws_col > 132 )
-            ws.ws_col = 132;
-
         fprintf( stderr, "\33[2J" );
         fflush( stdout );
     }
@@ -989,24 +982,22 @@ void set_monitor( char *interface, int channel )
     {
         /* possibly wlan-ng */
 
-        memset( s, 0, sizeof( s ) );
-        snprintf( s,  sizeof( s ) - 1, "channel=%d", channel );
+        sprintf( s, "channel=%d", channel );
 
         if( ! fork() )
         {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
+            close( 0 ); close( 1 ); close( 2 );
             execlp( "wlanctl-ng", "wlanctl-ng", interface,
                     "lnxreq_wlansniff", "enable=true",
                     "prismheader=true", "wlanheader=false",
-                    "stripfcs=true", "keepwepflags=true", s, NULL );
+                    "stripfcs=true", "keepwepflags=true", s, 0 );
             exit( 1 );
         }
 
         wait( NULL );
     }
 
-    memset( s, 0, sizeof( s ) );
-    snprintf( s,  sizeof( s ) - 1, "%d", channel );
+    sprintf( s, "%d", channel );
 
 #if 0
     /* orinoco monitor mode not enabled as ipw2200 *
@@ -1016,7 +1007,7 @@ void set_monitor( char *interface, int channel )
     {
         if( ! fork() )
         {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
+            close( 0 ); close( 1 ); close( 2 );
             execlp( "iwpriv", "iwpriv", interface,
                     "monitor", "1", s, 0 );
             exit( 1 );
@@ -1033,9 +1024,9 @@ void set_monitor( char *interface, int channel )
     {
         /* try the generic way */
 
-        close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
+        close( 0 ); close( 1 ); close( 2 );
         execlp( "iwconfig", "iwconfig", interface, "mode",
-                "Monitor", "channel", s, NULL );
+                "Monitor", "channel", s, 0 );
         exit( 1 );
     }
 
@@ -1048,32 +1039,30 @@ void set_channel( char *interface, int channel )
 
     if( memcmp( interface, "wlan", 4 ) == 0 )
     {
-        memset( s, 0, sizeof( s ) );
-        snprintf( s,  sizeof( s ) - 1, "channel=%d", channel );
+        sprintf( s, "channel=%d", channel );
 
         if( ! fork() )
         {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
+            close( 0 ); close( 1 ); close( 2 );
             execlp( "wlanctl-ng", "wlanctl-ng", interface,
                     "lnxreq_wlansniff", "enable=true",
                     "prismheader=true", "wlanheader=false",
-                    "stripfcs=true", "keepwepflags=true", s, NULL );
+                    "stripfcs=true", "keepwepflags=true", s, 0 );
             exit( 1 );
         }
 
         wait( NULL );
     }
 
-    memset( s, 0, sizeof( s ) );
-    snprintf( s,  sizeof( s ) - 1, "%d", channel );
+    sprintf( s, "%d", channel );
 
     if( memcmp( interface, "prism", 5 ) == 0 )
         interface = "eth1";
 
     if( ! fork() )
     {
-        close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-        execlp( "iwconfig", "iwconfig", interface, "channel", s, NULL );
+        close( 0 ); close( 1 ); close( 2 );
+        execlp( "iwconfig", "iwconfig", interface, "channel", s, 0 );
         exit( 1 );
     }
 
@@ -1085,7 +1074,7 @@ int bg_chans[] = { 7, 1, 5, 10, 13, 2, 6, 11, 12, 3, 8, 4, 9, 0 };
 char usage[] =
 
 "\n"
-"  airodump %d.%d - (C) 2004,2005 Christophe Devine\n"
+"  airodump 2.2 - (C) 2004,2005 Christophe Devine\n"
 "\n"
 "  usage: airodump <interface name or pcap filename>\n"
 "                  <output prefix> <channel> [IVs flag]\n"
@@ -1107,6 +1096,7 @@ int main( int argc, char *argv[] )
 {
     int n;
     int ivs_only;       /* if set, only save the unique WEP IVs */
+    int hip_hop;        /* if set, hop between b/g channels     */
     int power;          /* strength of signal if Prism2 header  */
     int fd_raw_in;      /* file descriptor of the raw socket    */
     int caplen;         /* length of captured packet            */
@@ -1132,7 +1122,7 @@ int main( int argc, char *argv[] )
     /* initialize a bunch of variables */
 
     ivs_only    =  0;
-    channel     =  0;
+    hip_hop     =  0;
     power       = -1;
     fd_raw_in   = -1;
     arptype     =  0;
@@ -1143,21 +1133,15 @@ int main( int argc, char *argv[] )
 
     if( argc < 4 || argc > 5 )
     {
-        printf( usage, _VER_MAJOR, _VER_MINOR );
+        printf( usage );
         return( 1 );
     }
 
-    channel  = atoi( argv[3] );
+    hip_hop  = atoi( argv[3] );
 
     if( argc == 5 )
         ivs_only = atoi( argv[4] );
 
-    /* create the raw socket and drop privileges */
-
-    fd_raw_in = socket( PF_PACKET, SOCK_RAW, htons( ETH_P_ALL ) );
-
-    setuid( getuid() );
-
     /* reserve the buffer space */
 
     if( ( buffer = (unsigned char *) malloc( 65536 ) ) == NULL )
@@ -1170,8 +1154,6 @@ int main( int argc, char *argv[] )
 
     if( ( f_cap_in = fopen( argv[1], "rb" ) ) != NULL )
     {
-        close( fd_raw_in );
-
         n = sizeof( struct pcap_file_header );
 
         if( fread( &pfh_in, 1, n, f_cap_in ) != (size_t) n )
@@ -1202,13 +1184,21 @@ int main( int argc, char *argv[] )
     }
     else
     {
-        if( fd_raw_in < 0 )
+        /* create the raw socket */
+
+        if( ( fd_raw_in = socket( PF_PACKET, SOCK_RAW,
+                                 htons( ETH_P_ALL ) ) ) < 0 )
         {
             perror( "socket(PF_PACKET) failed" );
-            fprintf( stderr, "This program requires root privileges.\n" );
+            if( getuid() != 0 )
+                fprintf( stderr, "This program requires root privileges.\n" );
             return( 1 );
         }
 
+        /* drop privileges */
+
+        setuid( getuid() );
+
         /* find the interface index */
 
         memset( &ifr, 0, sizeof( ifr ) );
@@ -1230,17 +1220,17 @@ int main( int argc, char *argv[] )
         {
             if( ! fork() )      /* wlan-ng brain damage */
             {
-                close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
+                close( 0 ); close( 1 ); close( 2 );
                 execlp( "wlanctl-ng", "wlanctl-ng", argv[1],
-                        "lnxreq_ifstate", "ifstate=enable", NULL );
+                        "lnxreq_ifstate", "ifstate=enable", 0 );
                 exit( 1 );
             }
             wait( NULL );
 
             if( ! fork() )      /* hostap card reset */
             {
-                close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-                execlp( "iwpriv", "iwpriv", argv[1], "reset", "1", NULL );
+                close( 0 ); close( 1 ); close( 2 );
+                execlp( "iwpriv", "iwpriv", argv[1], "reset", "1", 0 );
                 exit( 1 );
             }
             wait( NULL );
@@ -1276,74 +1266,55 @@ int main( int argc, char *argv[] )
 
         if( ! fork() )  /* hostap */
         {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", argv[1], "monitor_type", "1", NULL );
+            close( 0 ); close( 1 ); close( 2 );
+            execlp( "iwpriv", "iwpriv", argv[1], "monitor_type", "1", 0 );
             exit( 1 );
         }
         wait( NULL );
 
         if( ! fork() )  /* r8180 */
         {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", argv[1], "prismhdr", "1", NULL );
+            close( 0 ); close( 1 ); close( 2 );
+            execlp( "iwpriv", "iwpriv", argv[1], "prismhdr", "1", 0 );
             exit( 1 );
         }
         wait( NULL );
 
         if( ! fork() )  /* prism54 */
         {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwpriv", "iwpriv", argv[1], "set_prismhdr", "1", NULL );
-            exit( 1 );
-        }
-        wait( NULL );
-
-        /* make sure the WEP key is off */
-
-        if( ! fork() )
-        {
-            close( 0 ); close( 1 ); close( 2 ); chdir( "/" );
-            execlp( "iwconfig", "iwconfig", argv[1], "key", "off", NULL );
+            close( 0 ); close( 1 ); close( 2 );
+            execlp( "iwpriv", "iwpriv", argv[1], "set_prismhdr", "1", 0 );
             exit( 1 );
         }
         wait( NULL );
 
 skip_iwpriv:
 
-        /* start a child to hop between channels */
+        /* if hopping, start a child to hop between channels */
 
-        if( channel == 0 )
+        if( hip_hop == 0 )
         {
-            pipe( ch_pipe );
-
-            signal( SIGUSR1,  sighandler );
-
             if( ! fork() )
             {
                 int ch_idx = 0;
 
                 while( getppid() != 1 )
                 {
-                    channel = bg_chans[ch_idx++];
+                    set_channel( argv[1], bg_chans[ch_idx++] );
 
                     if( bg_chans[ch_idx] == 0 ) 
                         ch_idx = 0;
 
-                    set_channel( argv[1], channel );
-
-                    write( ch_pipe[1], &channel, sizeof( int ) );
-                    kill( getppid(), SIGUSR1 );
-
-                    usleep( 250000 );
+                    usleep( 300000 );
                 }
 
                 exit( 0 );
             }
 
-            usleep( 250000 );
+            usleep( 200000 );
         }
         else
-            set_channel( argv[1], channel );
+            set_channel( argv[1], hip_hop );
 
         /* lookup the hardware type */
 
@@ -1391,7 +1362,6 @@ skip_iwpriv:
         return( 1 );
 
     signal( SIGINT,   sighandler );
-    signal( SIGTERM,  sighandler );
     signal( SIGCHLD,  sighandler );
     signal( SIGWINCH, sighandler );
 
@@ -1478,8 +1448,7 @@ skip_iwpriv:
                 else
                 {
                     n = *(int *)( buffer + 4 );
-
-                    if( *(int *)( buffer + 0x44 ) != 0 )
+                    if( memcmp( argv[1], "prism", 5 ) == 0 )
                         power = *(int *)( buffer + 0x44 );
                     else
                         power = *(int *)( buffer + 0x5C );
@@ -1529,8 +1498,8 @@ skip_iwpriv:
 
         nb_pkt_read++;
 
-        dump_add_packet( h80211, caplen, power, channel,
-                         pkh_in.tv_sec, pkh_in.tv_usec );
+        dump_add_packet( h80211, caplen, power, pkh_in.tv_sec,
+                                                pkh_in.tv_usec );
 
         if( f_ivs_out != NULL ) fflush( f_ivs_out );
         if( f_cap_out != NULL ) fflush( f_cap_out );
